package com.example.kdan_data_center.datacenter.share

import com.example.kdan_data_center.datacenter.share.body.CreateShareLinkBody
import com.example.kdan_data_center.datacenter.share.data.*
import io.reactivex.Observable
import retrofit2.http.*

interface ShareLinkService {
    companion object {
        const val BASE_URL = "api/v3/share_links/"
    }

    @GET("list")
    fun getShareLinkList(
            @Header("Authorization") accessToken: String,
            @Query("page") page: Int?
    ): Observable<ShareLinkListData>

    @GET(".")
    fun getShareLinkInfo(
            @Header("Authorization") accessToken: String,
            @Query("url_token") urlToken: String
    ): Observable<ShareLinkData>

    @POST(".")
    fun createShareLink(
            @Header("Authorization") accessToken: String,
            @Header("Content-Type") contentType: String,
            @Body createShareLinkBody: CreateShareLinkBody
    ): Observable<CreateShareLinkData>

    @PUT(".")
    @FormUrlEncoded
    fun updateShareLink(
            @Header("Authorization") accessToken: String,
            @Field("url_token") urlToken: String,
            @Field("status") status: String
    ): Observable<UpdateShareLinkData>

    @HTTP(method = "DELETE", path = ".", hasBody = true)
    @FormUrlEncoded
    fun deleteShareLink(
            @Header("Authorization") accessToken: String,
            @Field("url_token") urlToken: String
    ): Observable<DeleteShareLinkData>

    @POST("download_folder")
    @FormUrlEncoded
    fun getDownloadLink(
            @Header("Authorization") accessToken: String,
            @Field("url_token") urlToken: String,
            @Field("email") email: String?
    ): Observable<DownloadLinkData>

    @GET("folder_token")
    fun getFolderToken(
            @Query("url_token") urlToken: String
    ): Observable<FolderTokenData>
}